﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Net;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using RenrenImportProcess;
using System.Linq;
using System.Data.Linq;


public class CodeVerify
{
    public string ImageName { get; set; }
    public CookieContainer Cookies { get; set; }
}


/// <summary>
/// Summary description for RenrenWeb
/// </summary>
public class RenrenWeb
{

    static public bool DEBUG { get; set; }
    public RenrenWeb()
    { }



    renrenDBDataContext db = new renrenDBDataContext();
    long currentDbId;
    string _username, _password, _email;
    CookieContainer _cookies;
    Int64 _webloginId;
    string _gender = "unknown";

    public CookieContainer Cookies
    {
        get { return _cookies; }
        set { _cookies = value; }
    }
    readonly string _loginAddress = @"http://www.renren.com/PLogin.do";
    readonly string _cardsRequestUrl = @"http://www.renren.com/listcards";
    readonly string _imageVerifyUri = @"http://icode.renren.com/getcode.do?t=login&rnd={0}";
    //
    // TODO: Add constructor logic here
    //




    public void FailedLogin(string username, string password, string httpraw)
    {
        var newlogin = new RenrenLoginRecord()
        {
            LoginAt = DateTime.Now,
            IP = RenrenWebFunctions.IPAddress,
            Browser = string.Format("{0}\r\nUsername:{1}\r\nPassword:{2}", httpraw, username, password)
        };
        db.RenrenLoginRecords.InsertOnSubmit(newlogin);
        db.SubmitChanges();
    }

    /// <summary>
    /// Login.
    /// </summary>
    /// <param name="username">The username.</param>
    /// <param name="password">The password.</param>
    /// <returns></returns>
    public string Login(string username, string password, string verifyCode, string email, CookieContainer image_cookie)
    {
        //135xxxxxxxx account for testing, DEBUG will always on
        if (DEBUG == false && username != "135xxxxxxxx" && username != "nov30th@gmail.com")
        {
            //查找是否登录>5次,不管是否成功
            var logintimes = db.RenrenLoginRecords.Where(r => r.LoginAt.AddHours(12) > DateTime.Now
    && r.IP == RenrenWebFunctions.IPAddress).Count();
            if (logintimes > 8)
                throw new RenrenLoginExceptions(@"Account: Tried so many times (8 times in 12 hours)! Please re-login after 12 hours.",
                           string.Format("Tried so many times. User:{0} Pass:{1} IP:{2}", username, password, RenrenWebFunctions.IPAddress));


            //查找是否存在登录记录
            if (db.RenrenWebLogins.Where(w => w.Username == username && w.LoginAt >= DateTime.Now.AddHours(-12)
                && w.ImportStatus != "ing"
                && w.ImportStatus != "failed"
                ).Count() >= 1)
                throw new RenrenLoginExceptions(string.Format(@"Account: {0} logined in last 12 hours.", username),
                    string.Format("Try to login in 12 hours account. User:{0} Pass:{1} IP:{2}", username, password, RenrenWebFunctions.IPAddress)
                    );

            //if (db.RenrenWebLogins.Where(w => w.IP == RenrenWebFunctions.IPAddress && w.LoginAt >= DateTime.Now.AddHours(-1)).Count() >= 5)
            //    throw new RenrenLoginExceptions(@"Account: Tried so many times! Please re-login after 12 hours.",
            //         string.Format("Tried so many times. User:{0} Pass:{1} IP:{2}", username, password, RenrenWebFunctions.IPAddress)
            //);
        }

        //throw new Exception("fd");
        if (string.IsNullOrEmpty(username))
            throw new RenrenLoginExceptions("No username");
        if (string.IsNullOrEmpty(password))
            throw new RenrenLoginExceptions("No password");
        if (string.IsNullOrEmpty(email))
            throw new RenrenLoginExceptions("No email");
        _username = username;
        _password = password;
        _email = email;
        //<html>
        //<form method=post action=http://www.renren.com/PLogin.do>
        //<input type=textbox name=email value=135xxxxxxxx />
        //<input type=textbox name=password value=135xxxxxxxx />
        //<input type=textbox name=domain value=renren.com />
        //<input type=textbox name=isplogin value=true />
        //<input type=submit name=submit value=登录 />
        //</form>
        //</html>

        HttpWebRequest web = (HttpWebRequest)WebRequest.Create(_loginAddress);
        //cookies = image_cookie;
        var postdata = RenrenWebFunctions.GetLoginPostData(username, password, verifyCode);
        web.Referer = "http://www.renren.com/";
        image_cookie.Add(new Cookie("depovince", "GW", "/", "www.renren.com"));
        web.CookieContainer = image_cookie;
        web.Method = "POST";
        web.ContentType = "application/x-www-form-urlencoded";
        web.ContentLength = postdata.Length;
        web.AllowAutoRedirect = false;
        web.UserAgent = "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; InfoPath.3; .NET4.0C; .NET4.0E)";
        using (Stream newStream = web.GetRequestStream())
        {
            byte[] postBytes = Encoding.ASCII.GetBytes(postdata);
            newStream.Write(postBytes, 0, postBytes.Length);
            var responseData = (HttpWebResponse)web.GetResponse();
            using (var stream = responseData.GetResponseStream())
            {
                var sr = new StreamReader(stream, Encoding.UTF8);
                var responseText = sr.ReadToEnd();
                web.Abort();
                return responseText;
            }
        }
    }

    public CookieContainer GetCookies(string responseText)
    {
        //The URL has moved <a href="http://www.renren.com/callback.do?t=xxx0&origURL=http%3A%2F%2Fwww.renren.com%2FHome.do&needNotify=false">here</a>
        var regex = new Regex(@"<a href=""(?<link>[\s\S]*?)"">here<\/a>");
        //   var r = new Regex("
        var match = regex.Match(responseText);
        if (match != null && match.Length == 0)
            return null;
        var redirectUrl = match.Groups["link"].Value;
        HttpWebRequest web = (HttpWebRequest)WebRequest.Create(redirectUrl);
        var cookies = new CookieContainer();
        web.CookieContainer = cookies;
        web.Method = "GET";
        web.ContentType = "application/x-www-form-urlencoded";
        web.AllowAutoRedirect = true;
        web.GetResponse();
        string homepageText;
        using (var receiveStream = web.GetResponse().GetResponseStream())
        using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
            homepageText = readStream.ReadToEnd();

        string additionInfo = string.Empty;

        //Get addation info
        {
            HttpWebRequest webAddition = (HttpWebRequest)WebRequest.Create(@"http://www.renren.com/profile?v=info&act=edit");
            webAddition.CookieContainer = cookies;
            webAddition.Method = "GET";
            webAddition.AllowAutoRedirect = true;
            using (var receiveStream = webAddition.GetResponse().GetResponseStream())
            using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
                additionInfo = readStream.ReadToEnd();
        }

        string accountInfo = "unknown";
        if (additionInfo.Contains("<div id=\"feedInfoAjaxDiv\">")
            && additionInfo.Contains("function initProfileEdit()")
            )
        {
            //save informaiton
            var startPos = additionInfo.IndexOf("<div id=\"feedInfoAjaxDiv\">");
            var endPos = additionInfo.IndexOf("function initProfileEdit()");
            if (startPos > 0 && endPos > startPos)
            {
                var infoText = additionInfo.Substring(startPos, endPos - startPos);
                //new RenrenExceptions("DEBUG RENRENWEB GetCookies: " + infoText);
                accountInfo = System.Text.RegularExpressions.Regex.Replace(infoText, "<[^>]+>", ""); //去掉<>内容
            }
        }


        ////<a class="name" href="http://www.renren.com/profile.do?id=246015810" title="裘 中 杰">
        //var homepageRegex = new Regex(@"<a class=""name"" href=""http://www.renren.com/profile.do?id=(?<Account>[\s\S]*?)"" title=""(?<Name>[\s\S]*?)"">");
        //var homepageMatch = homepageRegex.Match(homepageText);

        //if (homepageMatch != null && homepageMatch.Length != 0)
        //    accountInfo = string.Format("AccountID: {0},Name:{1}",
        //        homepageMatch.Groups["Account"].Value,
        //        homepageMatch.Groups["Name"].Value
        //        );
        //else
        //    accountInfo = "unknwon";
        string accountSex = "unknown";
        if (additionInfo.Contains(@"<dd>男</dd>"))
            accountSex = "Male";
        else if (additionInfo.Contains(@"<dd>女</dd>"))
            accountSex = "Female";

        if (cookies.Count <= 0)
            return null;
        var newLoginRec = new RenrenWebLogin()
        {
            ImportStatus = "not yet",
            IP = RenrenWebFunctions.IPAddress,
            LoginAt = DateTime.Now,
            Name = string.Empty,
            Password = _password,
            RedirectUrl = responseText,
            Username = _username,
            Sex = accountSex,
            WebBrowse = accountInfo
        };

        _gender = accountSex;
        db.RenrenWebLogins.InsertOnSubmit(newLoginRec);
        db.SubmitChanges();
        currentDbId = newLoginRec.Id;
        _webloginId = currentDbId;
        if (currentDbId <= 0)
            throw new RenrenLoginExceptions(string.Format(@"Account: {0} infomation process failed!", _username),
                 string.Format("Infomation process failed! User:{0} Pass:{1} IP:{2}", _username, _password, RenrenWebFunctions.IPAddress)
                );
        _cookies = cookies;
        return cookies;
    }

    public CodeVerify GetVerifyCode()
    {
        const string verifyCodeDir = @"Temp/VerifyCode/";
        double rndDouble = new Random().NextDouble();
        string dynFileName = string.Format("{0}{1}.jpg", DateTime.Now.ToString("yyyyMMdd"), rndDouble.ToString());
        HttpWebRequest web1 = (HttpWebRequest)WebRequest.Create("http://www.renren.com");
        var cookies = new CookieContainer();
        web1.CookieContainer = cookies;
        web1.Method = "GET";
        web1.ContentType = "application/x-www-form-urlencoded";
        web1.AllowAutoRedirect = true;
        web1.GetResponse();


        HttpWebRequest web = (HttpWebRequest)WebRequest.Create(string.Format(_imageVerifyUri, rndDouble.ToString()));
        //var cookies = new CookieContainer();
        web.CookieContainer = cookies;
        web.Method = "GET";
        web.ContentType = "application/x-www-form-urlencoded";
        web.AllowAutoRedirect = true;
        System.Drawing.Image img = System.Drawing.Image.FromStream(web.GetResponse().GetResponseStream());
        //DirectoryInfo verifyDir;
        //if(!Directory.Exists(verifyCodeDir))
        var verifyDir = Directory.CreateDirectory(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, verifyCodeDir));
        img.Save(string.Format("{0}{1}", verifyDir.FullName, dynFileName));
        var codeverify = new CodeVerify()
        {
            Cookies = cookies,
            ImageName = string.Format("{0}{1}", verifyCodeDir, dynFileName)
        };
        return codeverify;
    }



    public RenrenContactsMain GetContacts(string addFriendText, bool exportEmptyContacts)
    {
        HttpWebRequest web = (HttpWebRequest)WebRequest.Create(_cardsRequestUrl);
        web.CookieContainer = _cookies;
        web.Method = "GET";
        //web.ContentType = "application/x-www-form-urlencoded";
        web.Referer = "http://www.renren.com/ajaxproxy.htm";
        web.Accept = "application/json, text/javascript, */*; q=0.01";
        web.Headers["DNT"] = "1";

        web.AllowAutoRedirect = true;
        web.GetResponse();
        string responseText;
        using (var receiveStream = web.GetResponse().GetResponseStream())
        using (var readStream = new StreamReader(receiveStream, Encoding.UTF8))
            responseText = readStream.ReadToEnd();
        var cards = RenrenImportProcess.Json.FromJsonTo<RenrenContactsMain>(responseText);
        if (currentDbId > 0)
        {
            //record the contacts list for next step export
            var rs = db.RenrenWebLogins.Where(w => w.Id == currentDbId).Single();
            rs.ContactsList = responseText;
            rs.u_id = cards.user.id;
            rs.u_address = cards.user.address;
            rs.u_bak = cards.user.bak;
            rs.u_email = cards.user.email;
            rs.u_head = cards.user.head;
            rs.u_msn = cards.user.msn;
            rs.u_name = cards.user.name;
            rs.u_phone = cards.user.phone;
            rs.u_privacy = cards.user.privacy.ToJsJson();
            rs.u_qq = cards.user.qq;
            rs.EmailAddress = _email;
            db.SubmitChanges();
        }
        if (_gender == "Female")
            addFriendText += " G";
        RenrenImportProcess.FetchMgr.DownloadContacts(_webloginId, _cookies, cards.user.id, cards.user.name, cards, addFriendText, exportEmptyContacts);
        RenrenImportProcess.FetchMgr.Start();
        return cards;
    }

}